/************************************************************************************
 *  Launchpad demprogram: lcd_4x20
 *
 * 4x20 karakteres LCD modul vezrlse 4 bites mdban
 * A program kir egy rvid szveget, majd LED1-et villogtatja
 * kb. 2 Hz-es frekvencival.
 *
 * Az alaprtelmezett rajel belltsokat hasznljuk: 
 * ACLK = n/a, MCLK = SMCLK = DCO alaprtelmezett belltssal (~ 1 MHz)
 *
 * Hardver kvetelmnyek:
 *  - Launchpad MSP430G22x3 mikrovezrlvel
 *  - LCD 4x20 alfanumerikus kijelz, az albbi sszektssel: 
 *
 * -------------------------
 *  LCD modul     Launchpad
 * -------------------------
 *  1. GND -------  GND
 *  2. VCC -------  TP1(+5V) 
 *  3. VEE - 1 k -  GND
 *  4. RS  -------  P2.6/Xin
 *  5. R/W -------  GND
 *  6. E   -------  P2.7/Xout
 *  7. DB0          ---
 *  8. DB1          ---
 *  9. DB2          ---
 * 10. DB3          ---  
 * 11. DB4 -------  P1.4
 * 12. DB5 -------  P1.5
 * 13. DB6 -------  P1.6
 * 14. DB7 -------  P1.7
 *
 *  I. Cserny
 *  MTA ATOMKI, Debrecen
 *  May 2012
 *  Fejleszti krnyezet: IAR Embedded Workbench for MSP430 v5.30.1 Kickstart Edition
 ************************************************************************************/

#include "io430.h"
#include "stdint.h"

/* LCD port s vezrl bitek megadsa */
#define LCD_PORT     P1OUT
#define LCD_PORT_DIR P1DIR
#define LCD_MASK     BIT7+BIT6+BIT5+BIT4
#define LCD_RS       P2OUT_bit.P6
#define LCD_RS_DIR   P2DIR_bit.P6
#define LCD_E	     P2OUT_bit.P7
#define LCD_E_DIR    P2DIR_bit.P7


/**-----------------------------------------------
 * Inline fggvny, amely egy regiszter valamelyik 
 * bitcsoportjt atomi mvelettel mdostja
 *-----------------------------------------------
 * \param reg a mdostand regiszter neve
 * \param val a berand rtk (helyirtk helyesen!)
 * \param mask a mdostand bitcsoportot kijell maszk
 */
#define ChangeBits(reg,val,mask) reg^=((reg^val)&mask)

/**----------------------------------------------
 *   Ksleltet eljrs (1 - 65535 ms)
 *-----------------------------------------------
 * \param delay a ksleltets ms egysgben megadva
 */
void delay_ms(uint16_t delay) {
  uint16_t i;
  for(i=0; i<delay; i++) {
    __delay_cycles(1000);
  }
}

/**----------------------------------------------
 *   LCD Enable bemenet pulzlsa
 *-----------------------------------------------
 */
void lcd_toggle_E() {
  LCD_E = 1; 
    __delay_cycles(500);
  LCD_E = 0;
    __delay_cycles(500);
}  

/**----------------------------------------------
 *   Egy bjt kikldse az LCD vezrljnek
 *-----------------------------------------------
 * \param val a kirand rtk
 * \param cmd regisztervlaszt bit (0: parancs, 1: adat)
 */
void lcd_write(uint8_t val, uint8_t cmd) {
  LCD_RS = cmd;
  ChangeBits(LCD_PORT,(val&0xF0),LCD_MASK);
  lcd_toggle_E();
  ChangeBits(LCD_PORT,((val&0x0F)<<4),LCD_MASK);
  lcd_toggle_E();
  __delay_cycles(1000);
}

/*-----------------------------------------------
 *   LCD kijelz inicializlsa
 *-----------------------------------------------
 */
void lcd_init (void){
//-- Az LCD port inicializlsa
  ChangeBits(LCD_PORT,0x00,LCD_MASK);
  ChangeBits(LCD_PORT_DIR,LCD_MASK,LCD_MASK);
  P2SEL = 0;
  LCD_RS = 0;
  LCD_RS_DIR = 1;
  LCD_E = 0; 
  LCD_E_DIR = 1; 
  delay_ms(100);                      //40ms vrakozs bekapcsols utn
//-- Az LCD vezrljnek inicializlsa:
//-- 1. szoftveres LCD reset: 0x30 (8-bites md) kirsa hromszor
//-- 2. 4-bites zemmd belltsa
//-- 3. Kpernytrls, kurzor kikapcsolsa
  ChangeBits(LCD_PORT,0x30,LCD_MASK); 
  lcd_toggle_E ();
  delay_ms(5);                        //vrjunk legalbb 4.1ms-ot
  lcd_toggle_E ();
  __delay_cycles(100);                //vrjunk legalbb 100us-ot
  lcd_toggle_E ();
  __delay_cycles(40);                 //vrjunk legalbb 37us-ot
  ChangeBits(LCD_PORT,0x20,LCD_MASK); //0x20 = 4-bites zemmd
  lcd_toggle_E ();
  delay_ms(5);                        //vrjunk legalbb 4.1ms-ot
//-- Innen kezdve minden bjtot kt flbjtknt kell kirni! ------
  lcd_write(0x28,0);                  //display md bellts
  delay_ms(5);                        //vrjunk legalbb 4.1ms-ot
  lcd_write(0x08,0);                  //display letilts
  lcd_write(0x01,0);                  //kpernytrls
  lcd_write(0x0C,0);                  //display be, cursor, s villogs ki
}

/**----------------------------------------------
 *  Karakterfzr kirsa az LCD-re
 *-----------------------------------------------
* \param p_str karakterfzr mutat (nullval lezrt stringre mutat)
 */
void lcd_puts(char* p_str) {
  char c;
  while ((c=*p_str)) {
      lcd_write(c,1);
      p_str++;
  }
}

#define GOTO_LINE1() lcd_write(0x80,0)
#define GOTO_LINE2() lcd_write(0xC0,0)
#define GOTO_LINE3() lcd_write(0x94,0)
#define GOTO_LINE4() lcd_write(0xD4,0)

int main(void) {
  WDTCTL = WDTPW + WDTHOLD;           //watchdog letiltsa
  P1DIR |= BIT0;
  lcd_init();
  lcd_puts("<==== 4x20 LCD ====>");
  GOTO_LINE2();                       //A msodik sor elejre lp
  lcd_puts("--Now in 4bit mode--");
  GOTO_LINE4();
  lcd_puts("MCU: TI MSP430G2231 ");
  GOTO_LINE3();                       //A msodik sor elejre lp
  lcd_puts("Launchpad demo board");
  while(1) {
      P1OUT ^= BIT0;                  //LED1 villogtatsa
    __delay_cycles(250000);
  }
}     